@@ -89,12 +89,6 @@ static void process_dir PARAMS((char *pathname, char *name, int pathlen, struct
 /* Name this program was run with. */
 char *program_name;
 
-/* All predicates for each path to process. */
-struct predicate *predicates;
-
-/* The last predicate allocated. */
-struct predicate *last_pred;
-
 /* A file descriptor open to the initial working directory.
    Doing it this way allows us to work when the i.w.d. has
    unreadable parents.  */
@@ -128,29 +122,18 @@ int
 main (int argc, char **argv)
 {
   int i;
-  const struct parser_table *entry_close, *entry_print, *entry_open;
-  const struct parser_table *parse_entry; /* Pointer to the parsing table entry for this expression. */
-  struct predicate *cur_pred;
-  char *predicate_name;		/* Name of predicate being parsed. */
   int end_of_leading_options = 0; /* First arg after any -H/-L etc. */
+  struct predicate *eval_tree;
 
-  
   program_name = argv[0];
+  state.exit_status = 0;
 
-  /* We call check_nofollow() before setlocale() because the numbers 
-   * for which we check (in the results of uname) definitiely have "."
-   * as the decimal point indicator even under locales for which that 
-   * is not normally true.   Hence atof() would do the wrong thing 
-   * if we call it after setlocale().
+  /* Set the option defaults before we do the the locale
+   * initialisation as check_nofollow() needs to be executed in the
+   * POSIX locale.
    */
-#ifdef O_NOFOLLOW
-  options.open_nofollow_available = check_nofollow();
-#else
-  options.open_nofollow_available = false;
-#endif
+  set_option_defaults(&options);
 
-  options.regex_options = RE_SYNTAX_EMACS;
-  
 #ifdef HAVE_SETLOCALE
   setlocale (LC_ALL, "");
 #endif
@@ -158,220 +141,22 @@ main (int argc, char **argv)
   textdomain (PACKAGE);
   atexit (close_stdout);
 
-  
-  if (isatty(0))
-    {
-      options.warnings = true;
-      options.literal_control_chars = false;
-    }
-  else
-    {
-      options.warnings = false;
-      options.literal_control_chars = false; /* may change */
-    }
-  
-  
-  predicates = NULL;
-  last_pred = NULL;
-  options.do_dir_first = true;
-  options.maxdepth = options.mindepth = -1;
-  options.start_time = time (NULL);
-  options.cur_day_start = options.start_time - DAYSECS;
-  options.full_days = false;
-  options.stay_on_filesystem = false;
-  options.ignore_readdir_race = false;
+  /* Check for -P, -H or -L options. */
+  end_of_leading_options = process_leading_options(argc, argv);
 
-  state.exit_status = 0;
-
-#if defined(DEBUG_STAT)
-  options.xstat = debug_stat;
-#endif /* !DEBUG_STAT */
-
-  if (getenv("POSIXLY_CORRECT"))
-    options.output_block_size = 512;
-  else
-    options.output_block_size = 1024;
-
-  if (getenv("FIND_BLOCK_SIZE"))
-    {
-      error (1, 0, _("The environment variable FIND_BLOCK_SIZE is not supported, the only thing that affects the block size is the POSIXLY_CORRECT environment variable"));
-    }
-
-#if LEAF_OPTIMISATION
-  /* The leaf optimisation is enabled. */
-  options.no_leaf_check = false;
-#else
-  /* The leaf optimisation is disabled. */
-  options.no_leaf_check = true;
-#endif
-
-  set_follow_state(SYMLINK_NEVER_DEREF); /* The default is equivalent to -P. */
+  if (options.debug_options & DebugStat)
+    options.xstat = debug_stat;
 
 #ifdef DEBUG
-  fprintf (stderr, "cur_day_start = %s", ctime (&options.cur_day_start));
+  fprintf (stderr, "cur_day_start = %s", ctime (&p->cur_day_start));
 #endif /* DEBUG */
 
-  /* Check for -P, -H or -L options. */
-  for (i=1; (end_of_leading_options = i) < argc; ++i)
-    {
-      if (0 == strcmp("-H", argv[i]))
-	{
-	  /* Meaning: dereference symbolic links on command line, but nowhere else. */
-	  set_follow_state(SYMLINK_DEREF_ARGSONLY);
-	}
-      else if (0 == strcmp("-L", argv[i]))
-	{
-	  /* Meaning: dereference all symbolic links. */
-	  set_follow_state(SYMLINK_ALWAYS_DEREF);
-	}
-      else if (0 == strcmp("-P", argv[i]))
-	{
-	  /* Meaning: never dereference symbolic links (default). */
-	  set_follow_state(SYMLINK_NEVER_DEREF);
-	}
-      else if (0 == strcmp("--", argv[i]))
-	{
-	  /* -- signifies the end of options. */
-	  end_of_leading_options = i+1;	/* Next time start with the next option */
-	  break;
-	}
-      else
-	{
-	  /* Hmm, must be one of 
-	   * (a) A path name
-	   * (b) A predicate
-	   */
-	  end_of_leading_options = i; /* Next time start with this option */
-	  break;
-	}
-    }
-
   /* We are now processing the part of the "find" command line 
    * after the -H/-L options (if any).
    */
+  eval_tree = build_expression_tree(argc, argv, end_of_leading_options);
 
-  /* fprintf(stderr, "rest: optind=%ld\n", (long)optind); */
   
-  /* Find where in ARGV the predicates begin. */
-  for (i = end_of_leading_options; i < argc && strchr ("-!(),", argv[i][0]) == NULL; i++)
-    {
-      /* fprintf(stderr, "Looks like %s is not a predicate\n", argv[i]); */
-      /* Do nothing. */ ;
-    }
-  
-  /* Enclose the expression in `( ... )' so a default -print will
-     apply to the whole expression. */
-  entry_open  = find_parser("(");
-  entry_close = find_parser(")");
-  entry_print = find_parser("print");
-  assert(entry_open  != NULL);
-  assert(entry_close != NULL);
-  assert(entry_print != NULL);
-  
-  parse_open (entry_open, argv, &argc);
-  parse_begin_user_args(argv, argc, last_pred, predicates);
-  pred_sanity_check(last_pred);
-  
-  /* Build the input order list. */
-  while (i < argc)
-    {
-      if (strchr ("-!(),", argv[i][0]) == NULL)
-	usage (_("paths must precede expression"));
-      predicate_name = argv[i];
-      parse_entry = find_parser (predicate_name);
-      if (parse_entry == NULL)
-	{
-	  /* Command line option not recognized */
-	  error (1, 0, _("invalid predicate `%s'"), predicate_name);
-	}
-      
-      i++;
-      if (!(*(parse_entry->parser_func)) (parse_entry, argv, &i))
-	{
-	  if (argv[i] == NULL)
-	    /* Command line option requires an argument */
-	    error (1, 0, _("missing argument to `%s'"), predicate_name);
-	  else
-	    error (1, 0, _("invalid argument `%s' to `%s'"),
-		   argv[i], predicate_name);
-	}
-
-      pred_sanity_check(last_pred);
-      pred_sanity_check(predicates); /* XXX: expensive */
-    }
-  parse_end_user_args(argv, argc, last_pred, predicates);
-  
-  if (predicates->pred_next == NULL)
-    {
-      /* No predicates that do something other than set a global variable
-	 were given; remove the unneeded initial `(' and add `-print'. */
-      cur_pred = predicates;
-      predicates = last_pred = predicates->pred_next;
-      free ((char *) cur_pred);
-      parse_print (entry_print, argv, &argc);
-      pred_sanity_check(last_pred); 
-      pred_sanity_check(predicates); /* XXX: expensive */
-    }
-  else if (!default_prints (predicates->pred_next))
-    {
-      /* One or more predicates that produce output were given;
-	 remove the unneeded initial `('. */
-      cur_pred = predicates;
-      predicates = predicates->pred_next;
-      pred_sanity_check(predicates); /* XXX: expensive */
-      free ((char *) cur_pred);
-    }
-  else
-    {
-      /* `( user-supplied-expression ) -print'. */
-      parse_close (entry_close, argv, &argc);
-      pred_sanity_check(last_pred);
-      parse_print (entry_print, argv, &argc);
-      pred_sanity_check(last_pred);
-      pred_sanity_check(predicates); /* XXX: expensive */
-    }
-
-#ifdef	DEBUG
-  fprintf (stderr, "Predicate List:\n");
-  print_list (stderr, predicates);
-#endif /* DEBUG */
-
-  /* do a sanity check */
-  pred_sanity_check(predicates);
-  
-  /* Done parsing the predicates.  Build the evaluation tree. */
-  cur_pred = predicates;
-  eval_tree = get_expr (&cur_pred, NO_PREC);
-
-  /* Check if we have any left-over predicates (this fixes
-   * Debian bug #185202).
-   */
-  if (cur_pred != NULL)
-    {
-      error (1, 0, _("unexpected extra predicate"));
-    }
-  
-#ifdef	DEBUG
-  fprintf (stderr, "Eval Tree:\n");
-  print_tree (stderr, eval_tree, 0);
-#endif /* DEBUG */
-
-  /* Rearrange the eval tree in optimal-predicate order. */
-  opt_expr (&eval_tree);
-
-  /* Determine the point, if any, at which to stat the file. */
-  mark_stat (eval_tree);
-  /* Determine the point, if any, at which to determine file type. */
-  mark_type (eval_tree);
-
-#ifdef DEBUG
-  fprintf (stderr, "Optimized Eval Tree:\n");
-  print_tree (stderr, eval_tree, 0);
-  fprintf (stderr, "Optimized command line:\n");
-  print_optlist(stderr, eval_tree);
-  fprintf(stderr, "\n");
-#endif /* DEBUG */
-
   /* safely_chdir() needs to check that it has ended up in the right place. 
    * To avoid bailing out when something gets automounted, it checks if 
    * the target directory appears to have had a directory mounted on it as
@@ -406,7 +191,7 @@ main (int argc, char **argv)
     error (1, errno, _("cannot get current directory"));
 
   /* If no paths are given, default to ".".  */
-  for (i = end_of_leading_options; i < argc && strchr ("-!(),", argv[i][0]) == NULL; i++)
+  for (i = end_of_leading_options; i < argc && !looks_like_expression(argv[i], true); i++)
     {
       process_top_path (argv[i], 0);
     }
@@ -845,10 +630,11 @@ safely_chdir_lstat(const char *dest,
 	      saved_errno = 0;	/* silence the error message */
 	      goto fail;
 	    }
-#endif	  
-#ifdef DEBUG_STAT
-	  fprintf(stderr, "safely_chdir(): chdir(\"%s\")\n", dest);
 #endif
+	  
+	  if (options.debug_options & DebugSearch)
+	    fprintf(stderr, "safely_chdir(): chdir(\"%s\")\n", dest);
+
 	  if (0 == chdir(dest))
 	    {
 	      /* check we ended up where we wanted to go */
@@ -983,8 +769,11 @@ safely_chdir_nofollow(const char *dest,
 		      boolean *did_stat)
 {
   int extraflags, fd;
-  extraflags = 0;
 
+  (void) direction;
+  (void) statbuf_dest;
+  
+  extraflags = 0;
   *did_stat = false;
   
   switch (symlink_follow_option)
@@ -1056,7 +845,7 @@ safely_chdir(const char *dest,
    * processed, do them now because they must be done in the same
    * directory.
    */
-  complete_pending_execdirs(eval_tree);
+  complete_pending_execdirs(get_eval_tree());
 
 #if defined(O_NOFOLLOW)
   if (options.open_nofollow_available)
@@ -1076,9 +865,8 @@ chdir_back (void)
   
   if (starting_desc < 0)
     {
-#ifdef DEBUG_STAT
-      fprintf(stderr, "chdir_back(): chdir(\"%s\")\n", starting_dir);
-#endif
+      if (options.debug_options & DebugSearch)
+	fprintf(stderr, "chdir_back(): chdir(\"%s\")\n", starting_dir);
       
 #ifdef STAT_MOUNTPOINTS
       /* We will need the mounted device list.  Get it now if we don't
@@ -1103,9 +891,9 @@ chdir_back (void)
     }
   else
     {
-#ifdef DEBUG_STAT
-      fprintf(stderr, "chdir_back(): chdir(<starting-point>)\n");
-#endif
+      if (options.debug_options & DebugSearch)
+	fprintf(stderr, "chdir_back(): chdir(<starting-point>)\n");
+
       if (fchdir (starting_desc) != 0)
 	error (1, errno, "%s", starting_dir);
     }
@@ -1193,8 +981,10 @@ static void do_process_top_dir(char *pathname,
 			       int mode,
 			       struct stat *pstat)
 {
+  (void) pstat;
+  
   process_path (pathname, base, false, ".", mode);
-  complete_pending_execdirs(eval_tree);
+  complete_pending_execdirs(get_eval_tree());
 }
 
 static void do_process_predicate(char *pathname,
@@ -1202,8 +992,10 @@ static void do_process_predicate(char *pathname,
 				 int mode,
 				 struct stat *pstat)
 {
+  (void) mode;
+  
   state.rel_pathname = base;
-  apply_predicate (pathname, pstat, eval_tree);
+  apply_predicate (pathname, pstat, get_eval_tree());
 }
 
 
@@ -1307,7 +1099,9 @@ process_path (char *pathname, char *name, boolean leaf, char *parent,
   struct stat stat_buf;
   static dev_t root_dev;	/* Device ID of current argument pathname. */
   int i;
+  struct predicate *eval_tree;
 
+  eval_tree = get_eval_tree();
   /* Assume it is a non-directory initially. */
   stat_buf.st_mode = 0;
   state.rel_pathname = name;
@@ -1370,10 +1164,9 @@ process_path (char *pathname, char *name, boolean leaf, char *parent,
   if (options.do_dir_first && state.curdepth >= options.mindepth)
     apply_predicate (pathname, &stat_buf, eval_tree);
 
-#ifdef DEBUG
-  fprintf(stderr, "pathname = %s, stop_at_current_level = %d\n",
-	  pathname, state.stop_at_current_level);
-#endif /* DEBUG */
+  if (options.debug_options & DebugSearch)
+    fprintf(stderr, "pathname = %s, stop_at_current_level = %d\n",
+	    pathname, state.stop_at_current_level);
   
   if (state.stop_at_current_level == false)
     /* Scan directory on disk. */
@@ -1418,7 +1211,7 @@ process_dir (char *pathname, char *name, int pathlen, struct stat *statp, char *
 {
   int subdirs_left;		/* Number of unexamined subdirs in PATHNAME. */
   boolean subdirs_unreliable;	/* if true, cannot use dir link count as subdir limif (if false, it may STILL be unreliable) */
-  int idx;			/* Which entry are we on? */
+  unsigned int idx;		/* Which entry are we on? */
   struct stat stat_buf;
 
   struct savedir_dirinfo *dirinfo;
@@ -1465,7 +1258,7 @@ process_dir (char *pathname, char *name, int pathlen, struct stat *statp, char *
        * yet been processed, do them now because they must be done in
        * the same directory.
        */
-      complete_pending_execdirs(eval_tree);
+      complete_pending_execdirs(get_eval_tree());
       
       if (strcmp (name, "."))
 	{
@@ -1557,7 +1350,7 @@ process_dir (char *pathname, char *name, int pathlen, struct stat *statp, char *
 		   * In the latter case, -noleaf should be used routinely.
 		   */
 		  error(0, 0, _("WARNING: Hard link count is wrong for %s: this may be a bug in your filesystem driver.  Automatically turning on find's -noleaf option.  Earlier results may have failed to include directories that should have been searched."),
-			parent);
+			pathname);
 		  state.exit_status = 1; /* We know the result is wrong, now */
 		  options.no_leaf_check = true;	/* Don't make same
 						   mistake again */
@@ -1592,8 +1385,7 @@ process_dir (char *pathname, char *name, int pathlen, struct stat *statp, char *
        * yet been processed, do them now because they must be done in
        * the same directory.
        */
-      complete_pending_execdirs(eval_tree); 
-
+      complete_pending_execdirs(get_eval_tree()); 
 
       if (strcmp (name, "."))
 	{
