@@ -1,6 +1,6 @@
 /* find -- search for files in a directory hierarchy (fts version)
    Copyright (C) 1990, 91, 92, 93, 94, 2000, 
-                 2003, 2004, 2005 Free Software Foundation, Inc.
+                 2003, 2004, 2005, 2006 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -225,11 +225,13 @@ consider_visiting(FTS *p, FTSENT *ent)
   struct stat statbuf;
   mode_t mode;
   int ignore, isdir;
-  
+
   if (options.debug_options & DebugSearch)
     fprintf(stderr,
-	    "consider_visiting: end->fts_info=%s, ent->fts_path=%s\n",
+	    "consider_visiting: fts_info=%-6s, fts_level=%2d, "
+            "fts_path=%s\n",
 	    get_fts_info_name(ent->fts_info),
+            (int)ent->fts_level,
 	    quotearg_n_style(0, locale_quoting_style, ent->fts_path));
 
   /* Cope with various error conditions. */
@@ -279,9 +281,6 @@ consider_visiting(FTS *p, FTSENT *ent)
       state.type = mode = statbuf.st_mode;
     }
 
-  if (0 == ent->fts_level && (0u == state.starting_path_length))
-    state.starting_path_length = ent->fts_pathlen;
-
   if (mode)
     {
       if (!digest_mode(mode, ent->fts_path, ent->fts_name, &statbuf, 0))
@@ -294,7 +293,7 @@ consider_visiting(FTS *p, FTSENT *ent)
     || (FTS_D  == ent->fts_info)
     || (FTS_DP == ent->fts_info)
     || (FTS_DC == ent->fts_info);
-      
+
   if (isdir && (ent->fts_info == FTS_NSOK))
     {
       /* This is a directory, but fts did not stat it, so
@@ -306,12 +305,18 @@ consider_visiting(FTS *p, FTSENT *ent)
       return;
     }
 
-  if (options.maxdepth >= 0 && (ent->fts_level > options.maxdepth))
+  if (options.maxdepth >= 0)
     {
-      ignore = 1;
-      fts_set(p, ent, FTS_SKIP);
+      if (ent->fts_level >= options.maxdepth)
+	{
+	  fts_set(p, ent, FTS_SKIP); /* descend no further */
+	  
+	  if (ent->fts_level > options.maxdepth) 
+	    ignore = 1;		/* don't even look at this one */
+	}
     }
-  else if ( (ent->fts_info == FTS_D) && !options.do_dir_first )
+
+  if ( (ent->fts_info == FTS_D) && !options.do_dir_first )
     {
       /* this is the preorder visit, but user said -depth */ 
       ignore = 1;
@@ -330,8 +335,8 @@ consider_visiting(FTS *p, FTSENT *ent)
     {
       visit(p, ent, &statbuf);
     }
-	      
-	      
+
+
   if (ent->fts_info == FTS_DP)
     {
       /* we're leaving a directory. */
@@ -350,10 +355,20 @@ find(char *arg)
   FTSENT *ent;
   
 
+  state.starting_path_length = strlen(arg);
+
   arglist[0] = arg;
   arglist[1] = NULL;
   
   ftsoptions = FTS_NOSTAT;
+
+  /* Work around Savannah bug #17877, which manifests on systems which
+   * use the same inode number for more than one file (smbfs, FAT,
+   * sometimes some FUSE-based ones).  This happens for unreferenced
+   * files.  Fix suggested by Jim Meyering.
+   */
+  ftsoptions |= FTS_TIGHT_CYCLE_CHECK;
+  
   switch (options.symlink_handling)
     {
     case SYMLINK_ALWAYS_DEREF:
@@ -361,7 +376,7 @@ find(char *arg)
       break;
 	  
     case SYMLINK_DEREF_ARGSONLY:
-      ftsoptions |= FTS_COMFOLLOW;
+      ftsoptions |= FTS_COMFOLLOW|FTS_PHYSICAL;
       break;
 	  
     case SYMLINK_NEVER_DEREF:
@@ -403,9 +418,10 @@ process_all_startpoints(int argc, char *argv[])
   /* figure out how many start points there are */
   for (i = 0; i < argc && !looks_like_expression(argv[i], true); i++)
     {
+      state.starting_path_length = strlen(argv[i]);
       find(argv[i]);
     }
-  
+
   if (i == 0)
     {
       /* 
