@@ -1,6 +1,6 @@
 /* parser.c -- convert the command line args into an expression tree.
    Copyright (C) 1990, 1991, 1992, 1993, 1994, 2000, 2001, 2003, 
-                 2004, 2005 Free Software Foundation, Inc.
+                 2004, 2005, 2006 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -1403,7 +1403,7 @@ parse_wholename (const struct parser_table* entry, char **argv, int *arg_ptr)
 static boolean
 parse_perm (const struct parser_table* entry, char **argv, int *arg_ptr)
 {
-  mode_t perm_val;
+  mode_t perm_val[2];
   float rate;
   int mode_start = 0;
   boolean havekind = false;
@@ -1472,7 +1472,8 @@ parse_perm (const struct parser_table* entry, char **argv, int *arg_ptr)
       if (NULL == change)
 	error (1, 0, _("invalid mode `%s'"), argv[*arg_ptr]);
     }
-  perm_val = mode_adjust (0, change, 0);
+  perm_val[0] = mode_adjust (0, false, 0, change, NULL);
+  perm_val[1] = mode_adjust (0, true, 0, change, NULL);
   free (change);
   
   our_pred = insert_primary (entry);
@@ -1497,7 +1498,7 @@ parse_perm (const struct parser_table* entry, char **argv, int *arg_ptr)
 	  break;
 	}
     }
-  if (('/' == argv[*arg_ptr][0]) && (0 == perm_val))
+  if (('/' == argv[*arg_ptr][0]) && (0 == perm_val[0]) && (0 == perm_val[1]))
     {
       /* The meaning of -perm /000 will change in the future.
        * It currently matches no files, but like -perm -000 it
@@ -1512,7 +1513,7 @@ parse_perm (const struct parser_table* entry, char **argv, int *arg_ptr)
 	     argv[*arg_ptr]);
     }
 
-  our_pred->args.perm.val = perm_val & MODE_ALL;
+  memcpy (our_pred->args.perm.val, perm_val, sizeof perm_val);
   (*arg_ptr)++;
   return true;
 }
@@ -1641,8 +1642,8 @@ insert_regex (char **argv, int *arg_ptr, const struct parser_table *entry, int r
 				      re);
   if (error_message)
     error (1, 0, "%s", error_message);
-  (*arg_ptr)++;
   our_pred->est_success_rate = estimate_pattern_match_rate(argv[*arg_ptr], 1);
+  (*arg_ptr)++;
   return true;
 }
 
@@ -2180,7 +2181,12 @@ insert_fprintf (FILE *fp, const struct parser_table *entry, PRED_FUNC func, char
 	}
       else if (*scan == '%')
 	{
-	  if (scan[1] == '%')
+	  if (scan[1] == 0)
+	    {
+	      /* Trailing %.  We don't like those. */
+	      error (1, 0, _("error: %s at end of format string"), scan);
+	    }
+	  else if (scan[1] == '%')
 	    {
 	      segmentp = make_segment (segmentp, format, scan - format + 1,
 				       KIND_PLAIN, our_pred);
