@@ -1,21 +1,21 @@
 /* regexprops.c -- document the properties of the regular expressions 
                    understood by gnulib.
 
-   Copyright 2005, 2007 Free Software Foundation, Inc.
+   Copyright 2005 Free Software Foundation, Inc.
 
-   This program is free software: you can redistribute it and/or modify
+   This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation, either version 3 of the License, or
-   (at your option) any later version.
-   
+   the Free Software Foundation; either version 2, or (at your option)
+   any later version.
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 /* Written by James Youngman, <jay@gnu.org>. */
 
@@ -36,6 +36,8 @@ char *program_name;
 
 static void output(const char *s, int escape)
 {
+  (void) escape;
+  
   fputs(s, stdout);
 }
 
@@ -67,26 +69,16 @@ static void enum_item(const char *s)
   literal(s);
   newline();
 }
-static void table_item(const char *s)
-{
-  directive("@item");
-  newline();
-  content(s);
-  newline();
-}
-
-static void code(const char *s)
-{
-  directive("@code{");
-  content(s);
-  directive("}");
-}
 
 static void begin_subsection(const char *name,
 			  const char *next,
 			  const char *prev,
 			  const char *up)
 {
+  (void) next;
+  (void) prev;
+  (void) up;
+  
   newline();
   
   directive("@node ");
@@ -102,13 +94,6 @@ static void begin_subsection(const char *name,
   newline();
 }
 
-static void begintable_asis()
-{
-  newline();
-  directive("@table @asis");
-  newline();
-}
-
 static void begintable_markup(char const *markup)
 {
   newline();
@@ -144,7 +129,8 @@ static void newpara()
 }
 
 
-static int describe_regex_syntax(int options)
+static void
+describe_regex_syntax(int options)
 {
   newpara();
   content("The character @samp{.} matches any single character");
@@ -320,7 +306,7 @@ static int describe_regex_syntax(int options)
       literal("The character @samp{$} only represents the end of a string when it appears:");
       beginenum();
       enum_item("At the end of a regular expression");
-      enum_item("Before a close-group, signified by ");
+      enum_item("Before an close-group, signified by ");
       if (options & RE_NO_BK_PARENS)
 	{
 	  literal("@samp{)}");
@@ -435,7 +421,8 @@ static int describe_regex_syntax(int options)
 
 
 
-static int menu()
+static void
+menu(void)
 {
   int i, options;
   const char *name;
@@ -456,7 +443,8 @@ static int menu()
 }
 
 
-static int describe_all(const char *up)
+static void
+describe_all(const char *up)
 {
   const char *name, *next, *previous;
   int options;
