@@ -3,19 +3,20 @@
    Copyright 1990, 1997, 1998, 1999, 2000, 2001, 2003, 2004, 2005 Free
    Software Foundation, Inc.
 
-   This program is free software: you can redistribute it and/or modify
+   This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation, either version 3 of the License, or
-   (at your option) any later version.
-   
+   the Free Software Foundation; either version 2, or (at your option)
+   any later version.
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+
 /* Written by James Youngman, <jay@gnu.org>. */
 /* Derived from savedir.c, written by David MacKenzie <djm@gnu.org>. */
 
@@ -252,7 +253,6 @@ void free_dirinfo(struct savedir_dirinfo *p)
   p->entries = NULL;
   free(p->buffer);
   p->buffer = NULL;
-  free(p);
 }
 
 
@@ -263,7 +263,7 @@ new_savedirinfo (const char *dir, struct savedir_extrainfo **extra)
   struct savedir_dirinfo *p = xsavedir(dir, SavedirSort);
   char *buf, *s;
   size_t bufbytes = 0;
-  int i;
+  unsigned int i;
   
   if (p)
     {
