@@ -62,6 +62,7 @@
 
 #include <config.h>
 #include <stdio.h>
+#include <signal.h>
 #include <ctype.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -76,6 +77,10 @@
 /* We need <unistd.h> for isatty(). */
 #include <unistd.h>
 
+#if HAVE_FCNTL_H
+/* We use fcntl() */
+#include <fcntl.h>
+#endif
 
 #define NDEBUG
 #include <assert.h>
@@ -183,6 +188,12 @@ static int separator = '\n';
 static struct quoting_options * quote_opts = NULL;
 static bool stdout_is_a_tty;
 static bool print_quoted_filename;
+static bool results_were_filtered;
+
+static char*  slocate_db_pathname = "/var/lib/slocate/slocate.db";
+
+static const char *selected_secure_db = NULL;
+
 
 /* Read in a 16-bit int, high byte first (network byte order).  */
 
@@ -313,13 +324,15 @@ struct regular_expression
 struct process_data
 {
   int c;			/* An input byte.  */
+  char itemcount;	        /* Indicates we're at the beginning of an slocate db. */
   int count; /* The length of the prefix shared with the previous database entry.  */
   int len;
   char *original_filename;	/* The current input database entry. */
   size_t pathsize;		/* Amount allocated for it.  */
   char *munged_filename;	/* path or base_name(path) */
   FILE *fp;			/* The pathname database.  */
-  char *dbfile;			/* Its name, or "<stdin>" */
+  const char *dbfile;		/* Its name, or "<stdin>" */
+  int  slocatedb_format;	/* Allows us to cope with slocate's format variant */
   /* for the old database format,
      the first and second characters of the most common bigrams.  */
   char bigram1[128];
@@ -508,12 +521,37 @@ visit_locate02_format(struct process_data *procdata, void *context)
   int nread;
   (void) context;
 
-  if (procdata->c == LOCATEDB_ESCAPE)
-    procdata->count += (short)get_short (procdata->fp);
-  else if (procdata->c > 127)
-    procdata->count += procdata->c - 256;
+  if (procdata->slocatedb_format)
+    {
+      if (procdata->itemcount == 0)
+	{
+	  ungetc(procdata->c, procdata->fp);
+	  procdata->count = 0;
+	  procdata->len = 0;
+	}
+      else if (procdata->itemcount == 1)
+	{
+	  procdata->count = procdata->len-1;
+	}
+      else
+	{
+	  if (procdata->c == LOCATEDB_ESCAPE)
+	    procdata->count += (short)get_short (procdata->fp);
+	  else if (procdata->c > 127)
+	    procdata->count += procdata->c - 256;
+	  else
+	    procdata->count += procdata->c;
+	}
+    }
   else
-    procdata->count += procdata->c;
+    {
+      if (procdata->c == LOCATEDB_ESCAPE)
+	procdata->count += (short)get_short (procdata->fp);
+      else if (procdata->c > 127)
+	procdata->count += procdata->c - 256;
+      else
+	procdata->count += procdata->c;
+    }
 
   if (procdata->count > procdata->len || procdata->count < 0)
     {
@@ -538,6 +576,16 @@ visit_locate02_format(struct process_data *procdata, void *context)
 
   procdata->munged_filename = procdata->original_filename;
   
+  if (procdata->slocatedb_format)
+    {
+      /* Don't increment indefinitely, it might overflow. */
+      if (procdata->itemcount < 6)
+	{
+	  ++(procdata->itemcount);
+	}
+    }
+  
+
   return VISIT_CONTINUE;
 }
 
@@ -793,7 +841,9 @@ print_stats(int argc, size_t database_file_size)
 	 human_readable ((uintmax_t) database_file_size,
 			 hbuf, human_ceiling, 1, 1));
   
-  printf(_("Filenames: %s "),
+  printf( (results_were_filtered ? 
+	   _("Matching Filenames: %s ") :
+	   _("All Filenames: %s ")),
 	 human_readable (statistics.total_filename_count,
 			 hbuf, human_ceiling, 1, 1));
   printf(_("with a cumulative length of %s bytes"),
@@ -811,111 +861,251 @@ print_stats(int argc, size_t database_file_size)
 			 hbuf, human_ceiling, 1, 1));
   
   if (!argc)
-    printf(_("Compression ratio %4.2f%%\n"),
-	   100.0 * ((double)statistics.total_filename_length
-		    - (double) database_file_size)
-	   / (double) statistics.total_filename_length);
+    {
+      if (results_were_filtered)
+	{
+	  printf(_("Some filenames may have been filtered out, "
+		   "so we cannot compute the compression ratio.\n"));
+	}
+      else
+	{
+	  if (statistics.total_filename_length)
+	    {
+	      printf(_("Compression ratio %4.2f%%\n"),
+		     100.0 * ((double)statistics.total_filename_length
+			      - (double) database_file_size)
+		     / (double) statistics.total_filename_length);
+	    }
+	  else
+	    {
+	      printf(_("Compression ratio is undefined\n"));
+	    }
+	}
+    }
   printf("\n");
 }
 
+/*
+ * Return nonzero if the data we read in indicates that we are 
+ * looking at a LOCATE02 locate database. 
+ */
+static int 
+looking_at_gnu_locatedb (const char *data, size_t len)
+{
+  if (len < sizeof (LOCATEDB_MAGIC))
+    return 0;
+  else if (0 == memcmp (data, LOCATEDB_MAGIC, sizeof (LOCATEDB_MAGIC)))
+    return 1;			/* We saw the magic byte sequence */
+  else
+    return 0;
+}
+
+/*
+ * Return nonzero if the data we read in indicates that we are 
+ * looking at an slocate database. 
+ */
+static int 
+looking_at_slocate_locatedb (const char *filename,
+			     const char *data,
+			     size_t len,
+			     int *seclevel)
+{
+  char slocate_magic[] = "1";
+  size_t lenwanted = sizeof(slocate_magic);
+  assert(len <= 2);
+  
+  if (len < 2)
+    {
+      return 0;
+    }
+  else 
+    {
+      /* Check that the magic number is a one-byte string */
+      if (0 == data[1])
+	{
+	  if (isdigit((unsigned char)data[0]))
+	    {
+	      /* looks promising. */
+	      *seclevel = (data[0] - '0');
+	      
+	      if (*seclevel > 1)
+		{
+		  /* Hmm, well it's probably an slocate database 
+		   * of some awsomely huge security level, like 2.
+		   * We don't know how to handle those.
+		   */
+		  error(0, 0,
+			_("locate database `%s' looks like an slocate "
+			  "database but it seems to have security level %c, "
+			  "which GNU findutils does not currently support"),
+			filename, data[1]);
+		  return 1;
+		}
+	      else
+		{
+		  return 1;
+		}
+	    }
+	}
+      else
+	{
+	  /* Definitely not slocate. */
+	  return 0;
+	}
+    }
+}
 
 /* Print or count the entries in DBFILE that match shell globbing patterns in 
    ARGV. Return the number of entries matched. */
 
 static unsigned long
-locate (int argc,
-	char **argv,
-	char *dbfile,
-	int ignore_case,
-	int enable_print,
-	int basename_only,
-	int use_limit,
-	struct locate_limits *plimit,
-	int stats,
-	int op_and,
-	int regex,
-	int regex_options)
+search_one_database (int argc,
+		     char **argv,
+		     const char *dbfile,
+		     FILE *fp,
+		     off_t filesize,
+		     int ignore_case,
+		     int enable_print,
+		     int basename_only,
+		     int use_limit,
+		     struct locate_limits *plimit,
+		     int stats,
+		     int op_and,
+		     int regex,
+		     int regex_options)
 {
   char *pathpart; 		/* A pattern to consider. */
   int argn;			/* Index to current pattern in argv. */
   int need_fold;	/* Set when folding and any pattern is non-glob. */
   int nread;		     /* number of bytes read from an entry. */
   struct process_data procdata;	/* Storage for data shared with visitors. */
-  
-  int old_format = 0; /* true if reading a bigram-encoded database.  */
-  static bool did_stdin = false; /* Set to prevent rereading stdin. */
+  int slocate_seclevel;
   struct visitor* pvis; /* temp for determining past_pat_inspector. */
+  const char *format_name;
+  enum ExistenceCheckType do_check_existence;
+
+
+  /* We may turn on existence checking for a given database. 
+   * We ensure that we can return to the previous behaviour 
+   * by using two variables, do_check_existence (which we act on) 
+   * and check_existence (whcih indicates the default before we 
+   * adjust it on the bassis of what kind of database we;re using 
+   */
+  do_check_existence = check_existence;
+   
   
-  /* To check the age of the database.  */
-  struct stat st;
-  time_t now;
-
-
   if (ignore_case)
     regex_options |= RE_ICASE;
   
   procdata.len = procdata.count = 0;
-  if (!strcmp (dbfile, "-"))
-    {
-      if (did_stdin)
-	{
-	  error (0, 0, _("warning: the locate database can only be read from stdin once."));
-	  return 0;
-	}
-      
-	    
-      procdata.dbfile = "<stdin>";
-      procdata.fp = stdin;
-      did_stdin = true;
-    }
-  else
-    {
-      if (stat (dbfile, &st) || (procdata.fp = fopen (dbfile, "r")) == NULL)
-	{
-	  error (0, errno, "%s", dbfile);
-	  return 0;
-	}
-      time(&now);
-      if (now - st.st_mtime > WARN_SECONDS)
-	{
-	  /* For example:
-	     warning: database `fred' is more than 8 days old */
-	  error (0, 0, _("warning: database `%s' is more than %d %s old"),
-		 dbfile, WARN_NUMBER_UNITS, _(warn_name_units));
-	}
-      procdata.dbfile = dbfile;
-    }
-
-  procdata.pathsize = 1026;	/* Increased as necessary by locate_read_str.  */
-  procdata.original_filename = xmalloc (procdata.pathsize);
-
-  nread = fread (procdata.original_filename, 1, sizeof (LOCATEDB_MAGIC),
-		 procdata.fp);
-  if (nread != sizeof (LOCATEDB_MAGIC)
-      || memcmp (procdata.original_filename, LOCATEDB_MAGIC,
-		 sizeof (LOCATEDB_MAGIC)))
-    {
-      int i;
-      /* Read the list of the most common bigrams in the database.  */
-      nread = fread (procdata.original_filename + sizeof (LOCATEDB_MAGIC), 1,
-	  256 - sizeof (LOCATEDB_MAGIC), procdata.fp);
-      for (i = 0; i < 128; i++)
-	{
-	  procdata.bigram1[i] = procdata.original_filename[i << 1];
-	  procdata.bigram2[i] = procdata.original_filename[(i << 1) + 1];
-	}
-      old_format = 1;
-    }
+  procdata.slocatedb_format = 0;
+  procdata.itemcount = 0;
 
+  procdata.dbfile = dbfile;
+  procdata.fp = fp;
+    
   /* Set up the inspection regime */
   inspectors = NULL;
   lastinspector = NULL;
   past_pat_inspector = NULL;
+  results_were_filtered = false;
+  
+  procdata.pathsize = 1026;	/* Increased as necessary by locate_read_str.  */
+  procdata.original_filename = xmalloc (procdata.pathsize);
 
-  if (old_format)
-    add_visitor(visit_old_format, NULL);
-  else
-    add_visitor(visit_locate02_format, NULL);
+
+  nread = fread (procdata.original_filename, 1, SLOCATE_DB_MAGIC_LEN,
+		 procdata.fp);
+  if (looking_at_slocate_locatedb(procdata.dbfile,
+				  procdata.original_filename,
+				  nread,
+				  &slocate_seclevel))
+    {
+      error(0, 0,
+	    _("`%s' is an slocate database.  "
+	      "Support for these is new, expect problems for now "
+	      "(you are, after all, using the CVS code)."),
+	    procdata.dbfile);
+      
+      /* slocate also uses frcode, but with a different header. 
+       * We handle the header here and then work with the data 
+       * in the normal way.
+       */
+      if (slocate_seclevel > 1)
+	{
+	  /* We don't know what those security levels mean, 
+	   * so do nothing further 
+	   */
+	  return 0;
+	}
+      else if (slocate_seclevel > 0)
+	{
+	  /* Don't show the filenames to the user if they don't exist.
+	   * Showing stats is safe since filenames are only counted
+	   * after the existence check 
+	   */
+	  if (ACCEPT_NON_EXISTING == check_existence)
+	    {
+	      /* Do not allow the user to see a list of filenames that they 
+	       * cannot stat().
+	       */
+	      error(0, 0, 
+		    _("You specified the -E option, but that option "
+		      "cannot be used with slocate-format databases "
+		      "with a non-zero security level.  No results will be "
+		      "generated for this database.\n"));
+	      return 0;
+	    }
+	  if (ACCEPT_EXISTING != do_check_existence) 
+	    {
+	      if (enable_print || stats)
+		{
+		  error(0, 0,
+			_("`%s' is an slocate database.  "
+			  "Turning on the '-e' option."),
+			procdata.dbfile);
+		}
+	      do_check_existence = ACCEPT_EXISTING;
+	    }
+	}
+      add_visitor(visit_locate02_format, NULL);
+      format_name = "slocate";
+      procdata.slocatedb_format = 1;
+    }
+  else 
+    {
+      int nread2;
+      
+      procdata.slocatedb_format = 0;
+      nread2 = fread (procdata.original_filename+nread, 1, sizeof (LOCATEDB_MAGIC)-nread,
+		      procdata.fp);
+      if (looking_at_gnu_locatedb(procdata.original_filename, nread+nread2))
+	{
+	  add_visitor(visit_locate02_format, NULL);
+	  format_name = "GNU LOCATE02";
+	}
+      else				/* Use the old format */
+	{
+	  int i;
+
+	  nread += nread2;
+	  /* Read the list of the most common bigrams in the database.  */
+	  if (nread < 256)
+	    {
+	      int more_read = fread (procdata.original_filename + nread, 1,
+				     256 - nread, procdata.fp);
+	      /* XXX: check more_read+nread! */
+	    }
+	  
+	  for (i = 0; i < 128; i++)
+	    {
+	      procdata.bigram1[i] = procdata.original_filename[i << 1];
+	      procdata.bigram2[i] = procdata.original_filename[(i << 1) + 1];
+	    }
+	  format_name = "old";
+	  add_visitor(visit_old_format, NULL);
+	}
+    }
 
   if (basename_only)
     add_visitor(visit_basename, NULL);
@@ -941,6 +1131,7 @@ locate (int argc,
   /* Add an inspector for each pattern we're looking for. */
   for ( argn = 0; argn < argc; argn++ )
     {
+      results_were_filtered = true;
       pathpart = argv[argn];
       if (regex)
 	{
@@ -1005,9 +1196,10 @@ locate (int argc,
   /* We add visit_existing_*() as late as possible to reduce the
    * number of stat() calls.
    */
-  switch (check_existence)
+  switch (do_check_existence)
     {
       case ACCEPT_EXISTING:
+	results_were_filtered = true;
 	if (follow_symlinks)	/* -L, default */
 	  add_visitor(visit_existing_follow, NULL);
 	else			/* -P */
@@ -1015,6 +1207,7 @@ locate (int argc,
 	break;
 	  
       case ACCEPT_NON_EXISTING:
+	results_were_filtered = true;
 	if (follow_symlinks)	/* -L, default */
 	  add_visitor(visit_non_existing_follow, NULL);
 	else			/* -P */
@@ -1025,7 +1218,11 @@ locate (int argc,
 	/* do nothing; no extra processing. */
 	break;
     }
-      
+
+  /* Security issue: The stats visitor must be added immediately
+   * before the print visitor, because otherwise the -S option would
+   * leak information about files that the caller cannot see.
+   */
   if (stats)
     add_visitor(visit_stats, &statistics);
 
@@ -1059,7 +1256,7 @@ locate (int argc,
     {
 	printf(_("Database %s is in the %s format.\n"),
 	       procdata.dbfile,
-	       old_format ? _("old") : "LOCATE02");
+	       format_name);
     }
 
 
@@ -1075,7 +1272,8 @@ locate (int argc,
   
   if (stats)
     {
-      print_stats(argc, st.st_size);
+      if (filesize)
+	print_stats(argc, filesize);
     }
   
   if (ferror (procdata.fp))
@@ -1083,12 +1281,6 @@ locate (int argc,
       error (0, errno, "%s", procdata.dbfile);
       return 0;
     }
-  if (procdata.fp != stdin && fclose (procdata.fp) == EOF)
-    {
-      error (0, errno, "%s", dbfile);
-      return 0;
-    }
-
   return plimit->items_accepted;
 }
 
@@ -1146,8 +1338,80 @@ static struct option const longopts[] =
   {NULL, no_argument, NULL, 0}
 };
 
+
+static int 
+drop_privs(void)
+{
+  const char * what = "failed";
+  uid_t orig_euid = geteuid();
+
+  /* Use of setgroups() is restrcted to root only. */
+  if (0 == orig_euid)
+    {
+      gid_t groups[1];
+      groups[1] = getgid();
+      if (0 != setgroups(1, groups)) 
+	{
+	  what = _("failed to drop group privileges");
+	  goto fail;
+	}
+    }
+  
+  if (0 != setuid(getuid()))
+    {
+      what = _("failed to drop setuid privileges");
+      goto fail;
+    }
+  
+  /* Defend against the case where the attacker runs us with the
+   * capability to call setuid() turned off, which on some systems
+   * will cause the above attempt to drop privileges fail (leaving us
+   * privileged).
+   */
+  if (0 == setuid(0))
+    {
+      what = _("Failed to drop privileges");
+      goto fail;
+    }
+
+  /* success. */
+  return 0;
+  
+ fail:
+  error(1, errno, "%s", what);
+  abort();
+  kill(0, SIGKILL);
+  _exit(1);
+  /*NOTREACHED*/
+  /* ... we hope. */
+  for (;;)
+    {
+      /* deliberate infinite loop */
+    }
+}
+
+static int
+opendb(const char *name)
+{
+  int fd = open(name, O_RDONLY
+#if defined(O_LARGEFILE)
+		|O_LARGEFILE
+#endif
+		);
+  if (fd >= 0) 
+    {
+      /* Make sure it won't survive an exec */
+      if (0 != fcntl(fd, F_SETFD, FD_CLOEXEC)) 
+	{
+	  close(fd);
+	  fd = -1;
+	}
+    }
+  return fd;
+}
+
 int
-main (int argc, char **argv)
+dolocate (int argc, char **argv, int secure_db_fd)
 {
   char *dbpath;
   unsigned long int found = 0uL;
@@ -1161,7 +1425,10 @@ main (int argc, char **argv)
   int regex_options = RE_SYNTAX_EMACS;
   int stats = 0;
   int op_and = 0;
-  char *e;
+  const char *e;
+  FILE *fp;
+  int they_chose_db = 0;
+  bool did_stdin = false;	/* Set to prevent rereading stdin. */
   
   program_name = argv[0];
 
@@ -1178,9 +1445,15 @@ main (int argc, char **argv)
   quote_opts = clone_quoting_options (NULL);
   print_quoted_filename = true;
   
+  /* We cannot simultaneously trust $LOCATE_PATH and use the
+   * setuid-access-controlled database,, since that could cause a leak
+   * of private data.
+   */
   dbpath = getenv ("LOCATE_PATH");
-  if (dbpath == NULL)
-    dbpath = LOCATE_DB;
+  if (dbpath)
+    {
+      they_chose_db = 1;
+    }
 
   check_existence = ACCEPT_EITHER;
 
@@ -1206,6 +1479,7 @@ main (int argc, char **argv)
 
       case 'd':
 	dbpath = optarg;
+	they_chose_db = 1;
 	break;
 
       case 'e':
@@ -1286,6 +1560,19 @@ main (int argc, char **argv)
 	return 1;
       }
 
+
+  /* If the user gave the -d option or set LOCATE_PATH,
+   * relinquish access to the secure database.
+   */
+  if (they_chose_db)
+    {
+      if (secure_db_fd >= 0)
+	{
+	  close(secure_db_fd);
+	  secure_db_fd = 0;
+	}
+    }
+
   if (!just_count && !stats)
     print = 1;
 
@@ -1309,12 +1596,16 @@ main (int argc, char **argv)
   else
     stdout_is_a_tty = false;
 
-  next_element (dbpath, 0);	/* Initialize.  */
+  if (they_chose_db)
+    next_element (dbpath, 0);	/* Initialize.  */
 
   /* Bail out early if limit already reached. */
-  while ((e = next_element ((char *) NULL, 0)) != NULL  &&
-	 (!use_limit || limits.limit > limits.items_accepted))
+  while (!use_limit || limits.limit > limits.items_accepted)
     {
+      struct stat st;
+      int fd;
+      off_t filesize;
+      
       statistics.compressed_bytes = 
       statistics.total_filename_count = 
       statistics.total_filename_length = 
@@ -1322,16 +1613,113 @@ main (int argc, char **argv)
       statistics.newline_count = 
       statistics.highbit_filename_count = 0u;
 
-      if (0 == strlen(e) || 0 == strcmp(e, "."))
+      if (they_chose_db)
 	{
-	  /* Use the default database name instead (note: we
-	   * don't use 'dbpath' since that might itself contain a 
-	   * colon-separated list.
-	   */
-	  e = LOCATE_DB;
-	}
+	  /* Take the next element from the list of databases */
+	  e = next_element ((char *) NULL, 0);
+	  if (NULL == e)
+	    break;
 	  
-      found = locate (argc - optind, &argv[optind], e, ignore_case, print, basename_only, use_limit, &limits, stats, op_and, regex, regex_options);
+	  if (0 == strcmp (e, "-"))
+	    {
+	      if (did_stdin)
+		{
+		  error (0, 0,
+			 _("warning: the locate database can only be read from stdin once."));
+		  return 0;
+		}
+	      else
+		{
+		  e = "<stdin>";
+		  fd = 0;
+		  did_stdin = true;
+		}
+	    }
+	  else
+	    {
+	      if (0 == strlen(e) || 0 == strcmp(e, "."))
+		{
+		  e = LOCATE_DB;
+		}
+	  
+	      /* open the database */
+	      fd = opendb(e);
+	      if (fd < 0)
+		{
+		  error (0, errno, "%s", e);
+		  return 0;
+		}
+	    }
+	}
+      else
+	{
+	  if (-1 == secure_db_fd)
+	    {
+	      /* Already searched the database, it's time to exit the loop */
+	      break;
+	    }
+	  else
+	    {
+	      e = selected_secure_db;
+	      fd = secure_db_fd;
+	      secure_db_fd = -1;
+	    }
+	}
+      
+      /* Check the database to see if it is old. */
+      if (fstat(fd, &st))
+	{
+	  error (0, errno, "%s", e);
+	  /* continue anyway */
+	  filesize = (off_t)0;
+	}
+      else
+	{
+	  time_t now;
+	  
+	  filesize = st.st_size;
+	  
+	  if ((time_t)-1 == time(&now))
+	    {
+	      /* If we can't tell the time, we don't know how old the
+	       * database is.  But since the message is just advisory,
+	       * we continue anyway.
+	       */
+	      error (0, errno, "time system call");
+	    }
+	  else
+	    {
+	      if (now - st.st_mtime > WARN_SECONDS)
+		{
+		  /* For example:
+		     warning: database `fred' is more than 8 days old */
+		  error (0, 0,
+			 _("warning: database `%s' is more than %d %s old"),
+			 e, WARN_NUMBER_UNITS, _(warn_name_units));
+		}
+	    }
+	}
+
+      fp = fdopen(fd, "r");
+      if (NULL == fp)
+	{
+	  error (0, errno, "%s", e);
+	  return 0;
+	}
+
+      /* Search this database for all patterns simultaneously */
+      found = search_one_database (argc - optind, &argv[optind],
+				   e, fp, filesize,
+				   ignore_case, print, basename_only,
+				   use_limit, &limits, stats,
+				   op_and, regex, regex_options);
+
+      /* Close the databsase (even if it is stdin) */
+      if (fclose (fp) == EOF)
+	{
+	  error (0, errno, "%s", e);
+	  return 0;
+	}
     }
   
   if (just_count)
@@ -1344,3 +1732,37 @@ main (int argc, char **argv)
   else
     return 1;
 }
+
+#define ARRAYSIZE(a) (sizeof(a)/sizeof(a[0]))
+static int 
+open_secure_db(void)
+{
+  int fd, i;
+  
+  const char * secure_db_list[] = 
+    {
+      LOCATE_DB,
+      "/var/lib/slocate/slocate.db",
+      NULL
+    };
+  for (i=0; secure_db_list[i]; ++i)
+    {
+      fd = opendb(secure_db_list[i]);
+      if (fd >= 0)
+	{
+	  selected_secure_db = secure_db_list[i];
+	  return fd;
+	}
+    }
+  return -1;
+}
+
+int
+main (int argc, char **argv)
+{
+  int dbfd = open_secure_db();
+  drop_privs();
+  
+  return dolocate(argc, argv, dbfd);
+}
+
